/*
 * MQCM - MQSeries Code to Message conversion functions
 *
 * "mqcm.c" function module
 *
 * Environments: Win NT, HP-UX Unix, Sun Solaris Unix, OS/390
 *
 * Source file: L:\MQSeries\UnixUtilities\Source\mqcm-7.c
 */
#define ICPR "Copyright 2001 IBM Global Services Australia"
/*
 * Revision History
 * ================
 * Version  Date         Author    Changes
 *       1  20 Apr 1999  Glenn     Initial version for MQSeries 5.0
 *                       Baddeley <glenbadd@au1.ibm.com>
 *       2  14 Feb 2000  GB        Updated for MQSeries 5.1 using
 *                                 cmqc.h supplied for HP-UX,
 *                                 Make literals more compact
 *       3  28 Feb 2000  GB        Handle Reason Codes MQRC_using,
 *                                 Dont reference MQRC_APPL if MQVER50 defined
 *       4  01 May 2000  GB        Check MQRC_APPL defined before referencing
 *       5  22 Nov 2000  GB        Updated for MQSeries 5.2 using
 *                                 cmqc.h supplied for Windows NT,
 *                                 Change 2064-2081 to 2164-2181_UNDEFINED,
 *                                 Reduce storage usage for undefined codes,
 *                                 Make reentrant by accepting pointer to return
 *                                 strings, Document functions
 *       6  15 Feb 2001  GB        Add 6000-6129 Reason Codes for MQSeries 5.2 C++
 *       7  09 Oct 2001  GB        Change MQRC_PAGESET_FULL (for OS/390)
 *                                 to MQRC_STORAGE_MEDIUM_FULL (all platforms)
 */
#define IVER "@(#)MQCM.C VER=7 DATE=09-Oct-2001"

#include <string.h>
 
/* Define MQ data types and symbols */
#include <cmqc.h>

/* Define MQCM_ symbols */
#include "mqcmh.h"

/* Local data */
static char SrcIdent[] = IVER;
static char Copyright[] = ICPR;

static char *CcLits[] = 
 {
  "UNKNOWN",  /* -1 */
  "OK",
  "WARNING",
  "FAILED",   /* 2 */
 };

static char *RcLits2[] = 
 {
"ALIAS_BASE_Q_TYPE_ERROR",   /* 2001 */
"ALREADY_CONNECTED",
"BACKED_OUT",
"BUFFER_ERROR",
"BUFFER_LENGTH_ERROR",
"CHAR_ATTR_LENGTH_ERROR",
"CHAR_ATTRS_ERROR",
"CHAR_ATTRS_TOO_SHORT",
"CONNECTION_BROKEN",
"DATA_LENGTH_ERROR",
"DYNAMIC_Q_NAME_ERROR",
"ENVIRONMENT_ERROR",
"EXPIRY_ERROR",
"FEEDBACK_ERROR",
NULL,
"GET_INHIBITED",
"HANDLE_NOT_AVAILABLE",
"HCONN_ERROR",
"HOBJ_ERROR",
"INHIBIT_VALUE_ERROR",
"INT_ATTR_COUNT_ERROR",
"INT_ATTR_COUNT_TOO_SMALL",
"INT_ATTRS_ARRAY_ERROR",
"SYNCPOINT_LIMIT_REACHED",
"MAX_CONNS_LIMIT_REACHED",
"MD_ERROR",
"MISSING_REPLY_TO_Q",
NULL,
"MSG_TYPE_ERROR",
"MSG_TOO_BIG_FOR_Q",
"MSG_TOO_BIG_FOR_Q_MGR",
NULL,
"NO_MSG_AVAILABLE",
"NO_MSG_UNDER_CURSOR",
"NOT_AUTHORIZED",
"NOT_OPEN_FOR_BROWSE",
"NOT_OPEN_FOR_INPUT",
"NOT_OPEN_FOR_INQUIRE",
"NOT_OPEN_FOR_OUTPUT",
"NOT_OPEN_FOR_SET",
"OBJECT_CHANGED",
"OBJECT_IN_USE",
"OBJECT_TYPE_ERROR",
"OD_ERROR",
"OPTION_NOT_VALID_FOR_TYPE",
"OPTIONS_ERROR",
"PERSISTENCE_ERROR",
"PERSISTENT_NOT_ALLOWED",
"PRIORITY_EXCEEDS_MAXIMUM",
"PRIORITY_ERROR",
"PUT_INHIBITED",
"Q_DELETED",
"Q_FULL",
NULL,
"Q_NOT_EMPTY",
"Q_SPACE_NOT_AVAILABLE",
"Q_TYPE_ERROR",
"Q_MGR_NAME_ERROR",
"Q_MGR_NOT_AVAILABLE",
NULL,
"REPORT_OPTIONS_ERROR",
"SECOND_MARK_NOT_ALLOWED",
"SECURITY_ERROR",
NULL,
"SELECTOR_COUNT_ERROR",
"SELECTOR_LIMIT_EXCEEDED",
"SELECTOR_ERROR",
"SELECTOR_NOT_FOR_TYPE",
"SIGNAL_OUTSTANDING",
"SIGNAL_REQUEST_ACCEPTED",
"STORAGE_NOT_AVAILABLE",
"SYNCPOINT_NOT_AVAILABLE",
NULL,
NULL,
"TRIGGER_CONTROL_ERROR",
"TRIGGER_DEPTH_ERROR",
"TRIGGER_MSG_PRIORITY_ERR",
"TRIGGER_TYPE_ERROR",
"TRUNCATED_MSG_ACCEPTED",
"TRUNCATED_MSG_FAILED",
NULL,
"UNKNOWN_ALIAS_BASE_Q",
NULL,
NULL,
"UNKNOWN_OBJECT_NAME",
"UNKNOWN_OBJECT_Q_MGR",
"UNKNOWN_REMOTE_Q_MGR",
NULL,
NULL,
"WAIT_INTERVAL_ERROR",
"XMIT_Q_TYPE_ERROR",
"XMIT_Q_USAGE_ERROR",
"NOT_OPEN_FOR_PASS_ALL",
"NOT_OPEN_FOR_PASS_IDENT",
"NOT_OPEN_FOR_SET_ALL",
"NOT_OPEN_FOR_SET_IDENT",
"CONTEXT_HANDLE_ERROR",
"CONTEXT_NOT_AVAILABLE",
"SIGNAL1_ERROR",
"OBJECT_ALREADY_EXISTS",     /* 2100 */
"OBJECT_DAMAGED",
"RESOURCE_PROBLEM",
"ANOTHER_Q_MGR_CONNECTED",
"UNKNOWN_REPORT_OPTION",
"STORAGE_CLASS_ERROR",
"COD_NOT_VALID_FOR_XCF_Q",
"XWAIT_CANCELED",
"XWAIT_ERROR",
"SUPPRESSED_BY_EXIT",
"FORMAT_ERROR",
"SOURCE_CCSID_ERROR",
"SOURCE_INTEGER_ENC_ERROR",
"SOURCE_DECIMAL_ENC_ERROR",
"SOURCE_FLOAT_ENC_ERROR",
"TARGET_CCSID_ERROR",
"TARGET_INTEGER_ENC_ERROR",
"TARGET_DECIMAL_ENC_ERROR",
"TARGET_FLOAT_ENC_ERROR",
"NOT_CONVERTED",
"TRUNCATED",
"NO_EXTERNAL_PARTICIPANTS",
"PARTICIPANT_NOT_AVAILABLE",
"OUTCOME_MIXED",
"OUTCOME_PENDING",
"BRIDGE_STARTED",
"BRIDGE_STOPPED",
"ADAPTER_STORAGE_SHORTAGE",
"UOW_IN_PROGRESS",
"ADAPTER_CONN_LOAD_ERROR",
"ADAPTER_SERV_LOAD_ERROR",
"ADAPTER_DEFS_ERROR",
"ADAPTER_DEFS_LOAD_ERROR",
"ADAPTER_CONV_LOAD_ERROR",
"BO_ERROR",
"DH_ERROR",
"MULTIPLE_REASONS",
"OPEN_FAILED",
"ADAPTER_DISC_LOAD_ERROR",
"CNO_ERROR",
"CICS_WAIT_FAILED",
"DLH_ERROR",
"HEADER_ERROR",
"SOURCE_LENGTH_ERROR",
"TARGET_LENGTH_ERROR",
"SOURCE_BUFFER_ERROR",
"TARGET_BUFFER_ERROR",
NULL,
"IIH_ERROR",
"PCF_ERROR",
"DBCS_ERROR",
NULL,
"OBJECT_NAME_ERROR",
"OBJECT_Q_MGR_NAME_ERROR",
"RECS_PRESENT_ERROR",
"OBJECT_RECORDS_ERROR",
"RESPONSE_RECORDS_ERROR",
"ASID_MISMATCH",
"PMO_RECORD_FLAGS_ERROR",
"PUT_MSG_RECORDS_ERROR",
"CONN_ID_IN_USE",
"Q_MGR_QUIESCING",
"Q_MGR_STOPPING",
"DUPLICATE_RECOV_COORD",
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
"PMO_ERROR",
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
"API_EXIT_NOT_FOUND",
"API_EXIT_LOAD_ERROR",
"REMOTE_Q_NAME_ERROR",
"INCONSISTENT_PERSISTENCE",
"GMO_ERROR",
"CICS_BRIDGE_RESTRICTION",
"STOPPED_BY_CLUSTER_EXIT",
"CLUSTER_RESOLUTION_ERROR",
"CONVERTED_STRING_TOO_BIG",
"TMC_ERROR",
"STORAGE_MEDIUM_FULL",       /* 2192 also PAGESET_FULL on OS/390 */
"PAGESET_ERROR",
"NAME_NOT_VALID_FOR_TYPE",
"UNEXPECTED_ERROR",
"UNKNOWN_XMIT_Q",
"UNKNOWN_DEF_XMIT_Q",
"DEF_XMIT_Q_TYPE_ERROR",
"DEF_XMIT_Q_USAGE_ERROR",
NULL,                        /* 2200 */
"NAME_IN_USE",
"CONNECTION_QUIESCING",
"CONNECTION_STOPPING",
"ADAPTER_NOT_AVAILABLE",
NULL,
"MSG_ID_ERROR",
"CORREL_ID_ERROR",
"FILE_SYSTEM_ERROR",
"NO_MSG_LOCKED",
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
"FILE_NOT_AUDITED",
"CONNECTION_NOT_AUTHORIZED",
"MSG_TOO_BIG_FOR_CHANNEL",
"CALL_IN_PROGRESS",
"RMH_ERROR",
NULL,
"Q_MGR_ACTIVE",
"Q_MGR_NOT_ACTIVE",
"Q_DEPTH_HIGH",
"Q_DEPTH_LOW",
"Q_SERVICE_INTERVAL_HIGH",
"Q_SERVICE_INTERVAL_OK",
NULL,
NULL,
NULL,
NULL,
"UNIT_OF_WORK_NOT_STARTED",
"CHANNEL_AUTO_DEF_OK",
"CHANNEL_AUTO_DEF_ERROR",
"CFH_ERROR",
"CFIL_ERROR",
"CFIN_ERROR",
"CFSL_ERROR",
"CFST_ERROR",
NULL,
"INCOMPLETE_GROUP",
"INCOMPLETE_MSG",
"INCONSISTENT_CCSIDS",
"INCONSISTENT_ENCODINGS",
"INCONSISTENT_UOW",
"INVALID_MSG_UNDER_CURSOR",
"MATCH_OPTIONS_ERROR",
"MDE_ERROR",
"MSG_FLAGS_ERROR",
"MSG_SEQ_NUMBER_ERROR",
"OFFSET_ERROR",
"ORIGINAL_LENGTH_ERROR",
"SEGMENT_LENGTH_ZERO",
NULL,
"UOW_NOT_AVAILABLE",
"WRONG_GMO_VERSION",
"WRONG_MD_VERSION",
"GROUP_ID_ERROR",
"INCONSISTENT_BROWSE",
"XQH_ERROR",
"SRC_ENV_ERROR",
"SRC_NAME_ERROR",
"DEST_ENV_ERROR",
"DEST_NAME_ERROR",
"TM_ERROR",
"CLUSTER_EXIT_ERROR",
"CLUSTER_EXIT_LOAD_ERROR",
"CLUSTER_PUT_INHIBITED",
"CLUSTER_RESOURCE_ERROR",
"NO_DESTINATIONS_AVAILABLE",
"CONN_TAG_IN_USE",
"PARTIALLY_CONVERTED",
"CONNECTION_ERROR",
"OPTION_ENVIRONMENT_ERROR",
NULL,
NULL,
"CD_ERROR",
"CLIENT_CONN_ERROR",
"CHANNEL_STOPPED_BY_USER",
"HCONFIG_ERROR",
"FUNCTION_ERROR",
"CHANNEL_STARTED",
"CHANNEL_STOPPED",
"CHANNEL_CONV_ERROR",
"SERVICE_NOT_AVAILABLE",
"INITIALIZATION_FAILED",
"TERMINATION_FAILED",
"UNKNOWN_Q_NAME",
"SERVICE_ERROR",
"Q_ALREADY_EXISTS",
"USER_ID_NOT_AVAILABLE",
"UNKNOWN_ENTITY",
"UNKNOWN_AUTH_ENTITY",
"UNKNOWN_REF_OBJECT",
"CHANNEL_ACTIVATED",
"CHANNEL_NOT_ACTIVATED",     /* 2296, last in MQ 2.2 */
"UOW_CANCELED",
"FUNCTION_NOT_SUPPORTED",
"SELECTOR_TYPE_ERROR",
"COMMAND_TYPE_ERROR",        /* 2300 */
"MULTIPLE_INSTANCE_ERROR",
"SYSTEM_ITEM_NOT_ALTERABLE",
"BAG_CONVERSION_ERROR",
"SELECTOR_OUT_OF_RANGE",
"SELECTOR_NOT_UNIQUE",
"INDEX_NOT_PRESENT",
"STRING_ERROR",
"ENCODING_NOT_SUPPORTED",
"SELECTOR_NOT_PRESENT",
"OUT_SELECTOR_ERROR",
"STRING_TRUNCATED",
"SELECTOR_WRONG_TYPE",
"INCONSISTENT_ITEM_TYPE",
"INDEX_ERROR",
"SYSTEM_BAG_NOT_ALTERABLE",
"ITEM_COUNT_ERROR",
"FORMAT_NOT_SUPPORTED",
"SELECTOR_NOT_SUPPORTED",
"ITEM_VALUE_ERROR",
"HBAG_ERROR",
"PARAMETER_MISSING",
"CMD_SERVER_NOT_AVAILABLE",
"STRING_LENGTH_ERROR",
"INQUIRY_COMMAND_ERROR",
"NESTED_BAG_NOT_SUPPORTED",
"BAG_WRONG_TYPE",
"ITEM_TYPE_ERROR",
"SYSTEM_BAG_NOT_DELETABLE",
"SYSTEM_ITEM_NOT_DELETABLE",
"CODED_CHAR_SET_ID_ERROR",
"MSG_TOKEN_ERROR",
"MISSING_WIH",
"WIH_ERROR",
"RFH_ERROR",
"RFH_STRING_ERROR",          /* 2335, last in MQ 5.0 */
"RFH_COMMAND_ERROR",
"RFH_PARM_ERROR",
"RFH_DUPLICATE_PARM",
"RFH_PARM_MISSING",          /* 2339, last in MQ 5.1 */
"CHAR_CONVERSION_ERROR",
"UCS2_CONVERSION_ERROR",
"DB2_NOT_AVAILABLE",
"OBJECT_NOT_UNIQUE",
"CONN_TAG_NOT_RELEASED",
"CF_NOT_AVAILABLE",
"CF_STRUC_IN_USE",
"CF_STRUC_LIST_HDR_IN_USE",
"CF_STRUC_AUTH_FAILED",
"CF_STRUC_ERROR",
"CONN_TAG_NOT_USABLE",
"GLOBAL_UOW_CONFLICT",
"LOCAL_UOW_CONFLICT",
"HANDLE_IN_USE_FOR_UOW",
"UOW_ENLISTMENT_ERROR",
"UOW_MIX_NOT_SUPPORTED",
"WXP_ERROR",
"CURRENT_RECORD_ERROR",
"NEXT_OFFSET_ERROR",
"NO_RECORD_AVAILABLE",
"OBJECT_LEVEL_INCOMPATIBLE",
"NEXT_RECORD_ERROR"          /* 2361, last in MQ 5.2 */
 };

static char *RcLits6[] = 
 {
"REOPEN_EXCL_INPUT_ERROR",   /* 6100 */
"REOPEN_INQUIRE_ERROR", 
"REOPEN_SAVED_CONTEXT_ERR",
"REOPEN_TEMPORARY_Q_ERROR",
"ATTRIBUTE_LOCKED",
"CURSOR_NOT_VALID",
"ENCODING_ERROR",
"STRUC_ID_ERROR",
"NULL_POINTER",
"NO_CONNECTION_REFERENCE",
"NO_BUFFER",
"BINARY_DATA_LENGTH_ERROR",
"BUFFER_NOT_AUTOMATIC",
"INSUFFICIENT_BUFFER",
"INSUFFICIENT_DATA",
"DATA_TRUNCATED",
"ZERO_LENGTH",
"NEGATIVE_LENGTH",
"NEGATIVE_OFFSET",
"INCONSISTENT_FORMAT",
"INCONSISTENT_OBJECT_STATE",
"CONTEXT_OBJECT_NOT_VALID",
"CONTEXT_OPEN_ERROR",
"STRUC_LENGTH_ERROR",
"NOT_CONNECTED",
"NOT_OPEN",
"DISTRIBUTION_LIST_EMPTY",
"INCONSISTENT_OPEN_OPTIONS",
"WRONG_VERSION",
"REFERENCE_ERROR"            /* 6129, last in MQ 5.2 */ 
 };


/***************************************************************
 *
 * Function: MQCompCodeMsg
 *
 * Purpose:  Convert a MQSeries Completion Code to the
 *           symbolic name in a C string.  Returns
 *           "MQCC_nnnn_UNDEFINED" for undefined completion codes.
 *
 * Arguments:
 *  Name    Direction  Type     Purpose
 *
 *  Cc      IN         MQLONG   Completion Code
 *  RetStr  IN         * char   Buffer to contain symbolic name,
 *                              size must be at least 64 char
 *  return  OUT        * char   Symbolic name of the completion
 *                              code in null terminated string
 *
 ***************************************************************/

char *MQCompCodeMsg( MQLONG Cc, char *RetStr )
{
 if( (Cc >= MQCM_CCMIN) && (Cc <= MQCM_CCMAX) ) 
   sprintf( RetStr, "MQCC_%s", CcLits[ Cc + MQCM_CCOFFSET ] );
 else
   sprintf( RetStr, "MQCC_%d_UNDEFINED", Cc );

 return( RetStr );
}

/***************************************************************
 *
 * Function: MQReasonCodeMsg
 *
 * Purpose:  Convert a MQSeries Reason Code to the
 *           symbolic name in a C string.  Returns
 *           "MQRC_nnnn_UNDEFINED" for undefined reason codes.
 *
 * Arguments:
 *  Name    Direction  Type     Purpose
 *
 *  Rc      IN         MQLONG   Reason Code
 *  RetStr  IN         * char   Buffer to contain symbolic name,
 *                              size must be at least 64 char
 *  return  OUT        * char   Symbolic name of the reason
 *                              code in null terminated string
 *
 ***************************************************************/

char *MQReasonCodeMsg( MQLONG Rc, char *RetStr )
{
 if( Rc == MQRC_NONE )
   strcpy( RetStr, "MQRC_NONE" );

 else if( (Rc >= MQCM_RCMIN) && (Rc <= MQCM_RCMAX) ) 
   {
     if( RcLits2[ Rc + MQCM_RCOFFSET ] != NULL )
       sprintf( RetStr, "MQRC_%s", RcLits2[ Rc + MQCM_RCOFFSET ] );
     else
       sprintf( RetStr, "MQRC_%d_UNDEFINED", Rc );
   }

 else if( (Rc >= MQCM_RC6MIN) && (Rc <= MQCM_RC6MAX) ) 
   {
     if( RcLits6[ Rc + MQCM_RC6OFFSET ] != NULL )
       sprintf( RetStr, "MQRC_%s", RcLits6[ Rc + MQCM_RC6OFFSET ] );
     else
       sprintf( RetStr, "MQRC_%d_UNDEFINED", Rc );
   }

#ifdef MQRC_APPL_FIRST
 else if( Rc == MQRC_APPL_FIRST ) 
   strcpy( RetStr, "MQRC_APPL_FIRST" );
 else if( Rc == MQRC_APPL_LAST ) 
   strcpy( RetStr, "MQRC_APPL_LAST" );
 else if( (Rc > MQRC_APPL_FIRST) && (Rc < MQRC_APPL_LAST) ) 
   sprintf( RetStr, "MQRC_APPL_%d", Rc );
#endif

 else
   sprintf( RetStr, "MQRC_%d_UNDEFINED", Rc );

 return( RetStr );
}

